/* © Touch Base Media, 2015, All Rights Reserved */

angular.module('starter.services', [])


.factory('Config', function($q) {
  return {
    getValueForKey: function(key) {
        var deferred = $q.defer();
        value = [];
        Parse.Config.get().then(
          function(config) {
            console.log("Fetched config " + key + " from server.");
            var test = config.get(key);
            deferred.resolve(config.get(key));
          }, 
          function(error) {
            console.log("Failed to fetch. Using Cached Config.");
            var config = Parse.Config.current();
            deferred.resolve(config.get(key));
          });
        return deferred.promise;
    }
  };
})



.factory('UserData', function($q) {
    return {
      getValueForKey: function(key) {
          var deferred = $q.defer();
          var data = window.localStorage.getItem(key);
          deferred.resolve(data);
          return deferred.promise;
      },
  };
})



.factory('Translator', function($translate) {
    return {
      toggleAppLanguage: function() {
          var current = window.localStorage.getItem('currentLanguage');

          // Switch between english/french.
          if(current == 'en'){
            $translate.use('fr');
            window.localStorage.setItem('currentLanguage', 'fr');
          }
          else{
            $translate.use('en');
            window.localStorage.setItem('currentLanguage', 'en');
          }

      },
      getCurrentLanguage: function() {
        return window.localStorage.getItem('currentLanguage');
      }
  };
})



// Service for determining if data should be fetched again from the server.
.factory('RefreshLocalDataService', function($q, Config, UserData) {
    return {
      shouldRefreshLocalDataStorage: function(lastDataFetchStorageKey) {
         var deferred = $q.defer()
           return Config.getValueForKey("RefreshLocalDataRate").then(function (refreshRateHours){
               return UserData.getValueForKey(lastDataFetchStorageKey).then(function (lastFetchDate){
                if(lastFetchDate != null){
                    var hours = Math.floor(Date.now() - new Date(lastFetchDate)) / 36e5;
                    if(hours > refreshRateHours){
                      deferred.resolve(true);
                      return deferred.promise;
                    }
                    else{
                      deferred.resolve(false);
                      return deferred.promise;
                    }
                }
                else{
                  deferred.resolve(true);
                  return deferred.promise;
                }   
               });
         });
         
      }
  };
})


.factory('Locations', function($q, $cordovaGeolocation, RefreshLocalDataService) {
  var objects = [];
  return {
    getAll: function(){
       var deferred = $q.defer();
       var data = window.localStorage.getItem('locations');

       return RefreshLocalDataService.shouldRefreshLocalDataStorage("lastLocationFetch").then(function(shouldFetchNewData){
        if (!shouldFetchNewData && data != null) {
              objects = JSON.parse(data);
              console.log('using local location storage');
              deferred.resolve(objects);
        }
        else{
             var parseObject = Parse.Object.extend("Location");
             var query = new Parse.Query(parseObject);      
             query.equalTo("District", window.localStorage.getItem("usersDistrict"));
             query.find().then(
              function(results) {
                objects = [];
                for (var i = 0; i < results.length ; ++i) {
                 var obj = results[i];
                    objects.push({ 
                    objectId :  obj.id,
                    district : obj.attributes.District,
                    community:  obj.attributes.Community,
                    address:  obj.attributes.Address
                    });
                  }
                  console.log('Fetched locations from Parse');
                  window.localStorage.setItem('locations', JSON.stringify(objects));
                  window.localStorage.setItem('lastLocationFetch', new Date());
                  deferred.resolve(objects);
              },
              function(error) {
                navigator.notification.alert("Limited Internet Connection");
                deferred.reject(error);
              });
        }
        return deferred.promise; 
      });
    },
    get: function(objectId) {
      for (var i = 0; i < objects.length; i++) {
        if (objects[i].objectId == objectId) {
          return objects[i];
        }
      }
      return null;
    }
  };
})




.factory('Candidates', function($q, UserData, RefreshLocalDataService) {
  var getParseImageUrl = function(object){
      var imageFile = object.attributes.Photo;
      if(imageFile){
        var imageURL = imageFile.url();
        return imageURL;
      }
      else{
        return "img/parties/default.png"; // Default image if none provided
      }
  }
  var getUsersCandidateRating = function(candidate_id){
    if(!window.localStorage.getItem("candidateRating:" + candidate_id)){
      return null;
    }
    else{
      return window.localStorage.getItem("candidateRating:" + candidate_id);
    }
  }
  var getHasBeenRated = function(candidate_id){
    if(!window.localStorage.getItem("candidateRating:" + candidate_id)){
      return false;
    }
    else{
      return true;
    }
  }

  var shuffle = function (o){
    for(var j, x, i = o.length; i; j = Math.floor(Math.random() * i), x = o[--i], o[i] = o[j], o[j] = x);
    return o;
  }

  var objects = [];

  return {
    getAll: function(){
       var deferred = $q.defer();
       var data = window.localStorage.getItem('candidates');

       return RefreshLocalDataService.shouldRefreshLocalDataStorage("lastCandidateFetch").then(function(shouldFetchNewData){

        if (!shouldFetchNewData && data != null) {
              objects = JSON.parse(data);
              objects = shuffle(objects);
              console.log('using local candidate storage');
              deferred.resolve(objects);
        }
        else{
             var parseObject = Parse.Object.extend("Candidate");
             var query = new Parse.Query(parseObject);
             query.ascending("Name");  //specify sorting
             //query.limit(800);  //specify limit -- fetch only 20 objects
             query.equalTo("District", window.localStorage.getItem("usersDistrict"));
              query.find().then(
              function(results) {
                objects = [];
                for (var i = 0; i < results.length ; ++i) {
                 var obj = results[i];
                    console.log("image url:" + window.localStorage.getItem("PartyImageURL-" + obj.attributes.Party));
                      objects.push({ 
                            objectId :  obj.id,
                            name : obj.attributes.Name,
                            party :  obj.attributes.Party,
                            partyLogo : window.localStorage.getItem("PartyImageURL-" + obj.attributes.Party),
                            photo : getParseImageUrl(obj),
                            biography_english : obj.attributes.Biography_English,
                            biography_french : obj.attributes.Biography_French,
                            twitterHandle_english : obj.attributes.TwitterHandle_English,
                            twitterHandle_french : obj.attributes.TwitterHandle_French,
                            usersRating : getUsersCandidateRating(obj.id),
                            hasBeenRated : getHasBeenRated(obj.id),
                            twitterHandle_candidate : obj.attributes.Candidate_Twitter_Handle,
                            candidate_FB_page : obj.attributes.Candidate_FB_Page             
                          });
               
                  }
                  console.log('Fetched candidates from Parse');
                  window.localStorage.setItem('candidates', JSON.stringify(objects));
                  window.localStorage.setItem('lastCandidateFetch', new Date());
                  objects = shuffle(objects);
                  deferred.resolve(objects);
              },
              function(error) {
                navigator.notification.alert("Limited Internet Connection");
                deferred.reject(error);
              });
        }
        return deferred.promise;
      });
    },
    get: function(objectId) {
      for (var i = 0; i < objects.length; i++) {
        if (objects[i].objectId == objectId) {
          return objects[i];
        }
      }
      return null;
    },

    loadPartyImageURLs: function(){
     var parseObject = Parse.Object.extend("Party");
     var query = new Parse.Query(parseObject);
     query.find().then(
     function(results) {
        objects = [];
        for (var i = 0; i < results.length ; ++i) {
         var obj = results[i];
           if(!window.localStorage.getItem("PartyImageURL-" + obj.attributes.Party_Name)){
              if(obj.attributes.Photo){
                console.log("set image: " + obj.attributes.Photo.url());
                 window.localStorage.setItem("PartyImageURL-" + obj.attributes.Party_Name , obj.attributes.Photo.url());
              } 
              else{
                 window.localStorage.setItem("PartyImageURL-" + obj.attributes.Party_Name , "img/parties/default.png");
              }
            }
          }        
      },
      function(error) {
        navigator.notification.alert("Limited Internet Connection");
      });
    }


    
  };
})



.factory('RequirementStatus', function($q) {
    return {
      initializeRequirementStatusStorage: function() {
        if(window.localStorage.getItem('isUserRegistered') == null){
          window.localStorage.setItem('isUserRegistered', 'false');
        }
        if(window.localStorage.getItem('isUsersVoteDatePicked') == null){
          window.localStorage.setItem('isUsersVoteDatePicked', 'false');
        }
        if(window.localStorage.getItem('isUsersIDReady') == null){
          window.localStorage.setItem('isUsersIDReady', 'false');
        }
        if(window.localStorage.getItem('isUsersStudentInfoReady') == null){
          window.localStorage.setItem('isUsersStudentInfoReady', 'false');
        }
        if(window.localStorage.getItem('isUserAwareOfSpecialCircumstances') == null){
          window.localStorage.setItem('isUserAwareOfSpecialCircumstances', 'false');
        }
        if(window.localStorage.getItem('userHasWatchedVideo') == null){
          window.localStorage.setItem('userHasWatchedVideo', 'false');
        }
      },
      isUserRegistered: function() {
        var deferred = $q.defer();
        deferred.resolve(window.localStorage.getItem('isUserRegistered'));
        return deferred.promise;
      },
      isUsersVoteDatePicked: function() {
        var deferred = $q.defer();
        deferred.resolve(window.localStorage.getItem('isUsersVoteDatePicked'));
        return deferred.promise;
      },
      isUsersIDReady: function() {
        var deferred = $q.defer();
        deferred.resolve(window.localStorage.getItem('isUsersIDReady'));
        return deferred.promise;
      },
      isUsersStudentInfoReady : function() {
        var deferred = $q.defer();
        deferred.resolve(window.localStorage.getItem('isUsersStudentInfoReady'));
        return deferred.promise;
      },
      isUserAwareOfSpecialCircumstances: function() {
        var deferred = $q.defer();
        deferred.resolve(window.localStorage.getItem('isUserAwareOfSpecialCircumstances'));
        return deferred.promise;
      },
      userHasWatchedVideo: function() {
        var deferred = $q.defer();
        deferred.resolve(window.localStorage.getItem('userHasWatchedVideo'));
        return deferred.promise;
      }
  };
})
   

   

.factory('Notifications', function($q, RefreshLocalDataService) {
  var objects = [];
  return {
    getAll: function(){
        var deferred = $q.defer();
        var data = window.localStorage.getItem('notifications');

       return RefreshLocalDataService.shouldRefreshLocalDataStorage("lastNotificationFetch").then(function(shouldFetchNewData){
        if (!shouldFetchNewData && data != null) {
              objects = JSON.parse(data);
              console.log('using local notification storage');
              deferred.resolve(objects);
        }
        else{
             var parseObject = Parse.Object.extend("Notification");
             var query = new Parse.Query(parseObject);
             query.descending("Date");  //specify sorting
             //query.limit(20);  //specify limit -- fetch only 20 objects
              query.find().then(
              function(results) {
                var objects = [];
                for (var i = 0; i < results.length ; ++i) {
                 var obj = results[i];
                    objects.push({ 
                    objectId :  obj.id,
                    title_english : obj.attributes.Title_English,
                    title_french: obj.attributes.Title_French,
                    message_english : obj.attributes.Message_English,
                    message_french : obj.attributes.Message_French,
                    date:  obj.attributes.Date
                    });
                  }
                  console.log('Fetched notifications from Parse');
                  window.localStorage.setItem('notifications', JSON.stringify(objects));  
                  window.localStorage.setItem('lastNotificationFetch', new Date());
                  deferred.resolve(objects);
              },
              function(error) {
                navigator.notification.alert("Limited Internet Connection");
                deferred.reject(error);
              });
        }
        return deferred.promise;
      });
    },
    get: function(objectId) {
      for (var i = 0; i < objects.length; i++) {
        if (objects[i].objectId == objectId) {
          return objects[i];
        }
      }
      return null;
    }
  };
})








.factory('UberDistricts', function($q, UserData, RefreshLocalDataService) {
  var objects = [];

  return {
    checkIfUberAvailableInMyDistrict: function(){
       var deferred = $q.defer();
       var data = window.localStorage.getItem('uberAvailable');

       return RefreshLocalDataService.shouldRefreshLocalDataStorage("lastUberAvailableFetch").then(function(shouldFetchNewData){

        if (!shouldFetchNewData && data != null) {
              objects = JSON.parse(data);
              objects = shuffle(objects);
              console.log('using local uberAvailability storage');
              deferred.resolve(objects);
        }
        else{
             var parseObject = Parse.Object.extend("District");
             var query = new Parse.Query(parseObject);
             query.equalTo("Name", window.localStorage.getItem("usersDistrict"));
             query.equalTo("Has_Uber", true);
              query.find().then(
              function(results) {
                if(results.length > 0){
                  deferred.resolve(true);
                }
                else{
                  deferred.resolve(false);
                }
                console.log('Fetched uberAvailable from Parse');
                window.localStorage.setItem('uberAvailable', JSON.stringify(objects));
                window.localStorage.setItem('lastUberAvailableFetch', new Date());

              },
              function(error) {
                navigator.notification.alert("Limited Internet Connection");
                deferred.reject(error);
              });
        }
        return deferred.promise;
      });
    }

  };
})




.factory('Districts', function($q, $http) {
    return {
      getDistrictFromLatLong: function(latitude, longitude){
        return $http.get("http://represent.opennorth.ca/boundaries/federal-electoral-districts-next-election/?contains="+latitude+","+longitude);
      }
    }
})



.factory('PostalCode', function($q) {

  return {
    GetDataFromPostalCode: function(postalCode){
       var objects = [];
       console.log("Searching for postal code: " + postalCode);
        var deferred = $q.defer();
             postalCode = postalCode.replace(/^[ ]+|[ ]+$/g,'')
             postalCode = postalCode.replace(/[`~!@#$%^&*()_|+\-=?;:'",.<>\{\}\[\]\\\/]/gi, '');
             postalCode = postalCode.replace(/\s+/g, '');
             postalCode = postalCode.toUpperCase();
             console.log("REGEX'd postal code: " + postalCode);
             
             var parseObject = Parse.Object.extend("PostalCode");
             var query = new Parse.Query(parseObject);
             //query.descending("createdAt");  //specify sorting
             query.limit(1);  //specify limit -- fetch only 20 objects
             query.equalTo("Code", postalCode.toString());

              query.find().then(
              function(results) {
                for (var i = 0; i < results.length ; ++i) {
                 var obj = results[i];
                    objects.push({ 
                      objectId :  obj.id,
                      code : obj.attributes.Code,
                      latitude : obj.attributes.Latitude,
                      longitude :  obj.attributes.Logitude,
                      community : obj.attributes.Community,
                      province : obj.attributes.Province
                    });
                  }
                  console.log('Fetched Postal Code from Parse = ' + JSON.stringify(objects));
                  deferred.resolve(objects);
              },
              function(error) {
                navigator.notification.alert("Postal code was not recognized");
                deferred.reject(error);
              });
        return deferred.promise;
    }
  };
});